//Code written by Yonatan Naamad 

#include "time.h"
#include "graph.h"
#include "stdlib.h"
#include "stdio.h"

/*completely connect the graph with weights in [0,1)*/
void connectGraph(Graph *g)
{
	int i,j;

	srand(time(NULL)); /*seed the random number generator with the time*/

	/*create an edge for all pairs of vertices*/
	for (i = 0; i < g->numVertices-1; i++)
		for (j = i; j < g->numVertices; j++)
		{
			Edge *e = malloc(sizeof(Edge));
			if (e == 0)
			{
				fprintf(stderr,"Not enough memory to create new edge.");
				exit(1);
			}

			/*set properties of the edge*/
			e->v1 = g->vertices[i];
			e->v2 = g->vertices[j];
			e->weight = ((double)rand() / ((double)(RAND_MAX)) ); /*(uniformly) random number in [0,1]*/

			/*notify the vertices of the new connection*/
			g->vertices[j]->edges[i] = e;
			g->vertices[i]->edges[j] = e;
		}
}

/*can be used for testing. This connects a graph with 6 vertices
 *as per the figure in DPV beneath the Prim's algorithm pseudocode.
 *Will not work unless the vertex count is 6.*/
void testConnectGraph(Graph *g)
{
	int i,j, k;
	int v1[10] =   {0,0,0,1,1,2,2,2,3,4};
	int v2[10] =   {1,2,3,2,3,3,4,5,5,5};
	double w[10] = {5,6,4,1,2,2,5,3,4,4};
	for (i = 0; i < 5; i++)
		for (j = i; j < 6; j++)
		{
			Edge *e = malloc(sizeof(Edge));

			/*set properties of the edge*/
			e->v1 = g->vertices[i];
			e->v2 = g->vertices[j];
			
			e->weight = 10000000.0; /*Set the edge to be prohibitively heavy. 
						 *We will later reweight if necessary*/
			/*notify the vertices of the new connection*/
			g->vertices[j]->edges[i] = e;
			g->vertices[i]->edges[j] = e;
			for (k = 0; k < 10; k++)
				if (i == v1[k] && j == v2[k]) e->weight = w[k];
		}
}

/*Create a graph with the given number of vertices*/
Graph * createGraph(int numVertices)
{
	int i;
	Graph *g = malloc(sizeof(Graph));
	g->vertices = malloc(sizeof(Vertex *)*numVertices); /*allocate room for vertex pointers*/
	if (g->vertices == 0)
	{
		fprintf(stderr,"Not enough memory to create vertex list.");
		exit(1);
	}
	g->numVertices = numVertices;
	for (i = 0; i < numVertices; i++)
	{
		g->vertices[i] = malloc(sizeof(Vertex)); /*create actual vertices*/
		if (g->vertices[i] == 0)
		{
			fprintf(stderr,"Not enough memory to create new vertex.");
			exit(1);
		}
		g->vertices[i]->id = i; /* give vertices some identifier by which they will later be recognized */
		g->vertices[i]->edges = malloc(sizeof(Edge *)*numVertices); /*initialize edge array*/
		if (g->vertices[i] == 0)
		{
			fprintf(stderr,"Not enough memory to create new edge array.");
			exit(1);
		}
	}
	return g;
}
