//Code written by Yonatan Naamad

#include "sys/timeb.h"
#include "time.h"
#include "stdio.h"
#include "stdlib.h"
#include "graph.h"
#include "disjoint-set.h"

#define  MAX_LEVELS  1000
#define SHOW_OUTPUT 0
#define INFINITY 10000000.0 /*any sufficiently large number should work*/
#define NONE (Vertex *)-1

/*used as the return value for the main algorithm*/
struct AR
{
	double withsort;
	double withoutsort;
	double mstsize;
};

/*merge two (Edge *) arrays (for mergesort) */
void merge(Edge **edgeList, int count1, int count2)
{
	int i=0,j=0,tc=count1+count2;
	Edge **r = edgeList+count1; //the right list (the 'count1'th element of edgeList onwards)
	Edge **l = edgeList; //the left list (the first count1 elements of edgeList)
	Edge **temp = malloc(sizeof(Edge *)*tc);
	
	while(count1 > 0 && count2 > 0)
	{
		/*keep picking the smallest element to put into temp*/
		if (l[0]->weight <= r[0]->weight)
		{
			temp[j++] = l[0];
			l++;
			count1--;
		}
		else
		{
			temp[j++] = r[0];
			r++;
			count2--;
		}
	}
	/*add remaining from left list*/
	while (count1 > 0)
	{
		temp[j++] = l[0];
		l++;
		count1--;
	}
	/*add remaining from right list*/
	while (count2 > 0)
	{
		temp[j++] = r[0];
		r++;
		count2--;
	}

	/*copy over temp into the original list*/
	for (i = 0; i < tc; i++)
		edgeList[i] = temp[i];

	free(temp);
}	

/*sort the edge list in order of increasing weights (using mergesort)*/
void mergeSort(Edge **edgeList, int count)
{
	if (count <= 1)
		return;
	mergeSort(edgeList,count/2);
	mergeSort(edgeList+count/2,count-count/2);
	merge(edgeList,count/2, count - count/2);

}

/*runs Kruskal's algorithm, and outputs the results if requested
 *returns the total edge weight in the resultant tree*/
struct AR *kruskal(int numVertices, int output, Graph *g)
{
  	double totalweight=0.0;
	double presort, postsort, postalgo; /*used for times*/
	struct timeb tp;
	struct AR *ret = malloc(sizeof(struct AR));
	// Write code here-(May need more declarations)

	/*get the time right before we begin the sorting DO NOT CHANGE*/
	ftime(&tp);
	presort = tp.time + .001*tp.millitm;

	// Write Code here (Calls sorting function and other code)

	/*get the time right after we finish the sorting DO NOT CHANGE*/	
	ftime(&tp);
	postsort = tp.time + .001*tp.millitm;
	
	/*Write Code here (Implement Kruskal's algorithm from DPV)*/
	

	/* Finish up and return. DO NOT CHANGE */
	ftime(&tp);
	postalgo = tp.time + .001*tp.millitm;
	ret->withsort = postalgo-presort;
	ret->withoutsort = postalgo-postsort;
	ret->mstsize = totalweight;
	return ret;
}

int main(int argc, char **argv)
{
	/* main takes 3 command line arguments:
	 * argv[1] = number of vertices in the graph (int)
	 * argv[2] = whether or not to print the tree (0 for no printing, any other int otherwise)
	 * argv[3] = whether or not to print out the average edge weight (0 for no printing, any other int otherwise)
	 * argv[4] = 0 for no time printing, 1 for print algorithm time with sort, 2 for algorithm without sort, 3 for both*/
	double average, time;
	int numVertices;
	struct AR *p;

	if (argc != 5)
	{
		fprintf(stderr, "Invalid number of arguments");
		return 1;
	}

	numVertices = atoi(argv[1]);
/*make and connect the graph*/
	Graph *g = createGraph(numVertices);
	connectGraph(g);
	p =  kruskal(numVertices, atoi(argv[2]),g);
	
	average = p->mstsize / (numVertices-1); /*the average is the total weight of the tree divided by the number of edges (numVertices - 1)*/		

	if (atoi(argv[3]))
		printf("Average edge weight in the MST: %lf\n", average);

	/*print times as requested*/	
	switch (atoi(argv[4]))
	{
		case 0:
			break;
		case 1:
			printf("With sort: %.3lf\n", p->withsort);
			break;
		case 2:
			printf("Without sort: %.3lf\n", p->withoutsort);
			break;
		case 3:
		default:
			printf("With sort: %.3lf\n", p->withsort);
			printf("Without sort: %.3lf\n", p->withoutsort);
			break;
	}
	return 0;
}
