//Code written by Yonatan Naamad

#include "heap.h"
#include "graph.h"
#include "stdio.h"
#include "stdlib.h"

/*initialize the heap with size numVertices*/
Heap * makeHeap(int numVertices)
{
	int i;
	Heap *h = malloc(sizeof(Heap *));
	h->heap = malloc(sizeof(Vertex *)*numVertices);
	if (h->heap == 0)
	{
		fprintf(stderr,"Not enough memory to create new heap array.");
		exit(1);
	}
	h->size = 0;
	for (i = 0; i < numVertices; i++)
		h->heap[i] = (Vertex *)0;
	return h;
}

/*update the location of the vertex in the heap (in accordance with value changes) */
void updateVertex(Heap *h, Vertex *v)
{
	int p = v->heappos;
	while (h->heap[p]->cost < h->heap[(p-1)/2]->cost)
	{
		Vertex *temp = h->heap[(p-1)/2];
		h->heap[(p-1)/2] = h->heap[p];
		h->heap[p] = temp;

		/*just telling each vertex where in the heap it currently is*/
		h->heap[(p-1)/2]->heappos = (p-1)/2;
		h->heap[p]->heappos = p;

		p = (p-1)/2;
	}
}

/*insert a vertex into the heap*/
void insert(Heap *h, Vertex *v)
{
	int p = h->size++;
	h->heap[p] = v;
	h->heap[p]->heappos = p;
	while (h->heap[p]->cost < h->heap[(p-1)/2]->cost)
	{
		Vertex *temp = h->heap[(p-1)/2];
		h->heap[(p-1)/2] = h->heap[p];
		h->heap[p] = temp;

		/*just telling each vertex where in the heap it currently is*/
		h->heap[(p-1)/2]->heappos = (p-1)/2;
		h->heap[p]->heappos = p;

		p = (p-1)/2;
	}
}

/*delete and return the top value of the heap*/
Vertex *deleteMin(Heap *h)
{
	int p=0;
	Vertex *ret = h->heap[0];  /*we will return the first element of the heap*/

	/*move the last element of the heap to the root and sift down*/
	h->heap[0] = h->heap[--(h->size)];
	h->heap[0]->heappos = 0;

	while ((2*p+1 < h->size && h->heap[p]->cost > h->heap[2*p+1]->cost) || (2*p+2 < h->size && h->heap[p]->cost > h->heap[2*p+2]->cost)) /*checks whether or not we will be sifting*/
	{	
		Vertex *temp;

		/*we want to swap with the lesser of the two children (or swap with the only child)*/
		if (2*p+1 == h->size || h->heap[2*p+1]->cost < h->heap[2*p+2]->cost)
		{
			temp = h->heap[2*p+1];
			h->heap[2*p+1] = h->heap[p];
			h->heap[p] = temp;

			/*just telling each vertex where in the heap it currently is*/
			h->heap[2*p+1]->heappos = 2*p+1;
			h->heap[p]->heappos = p;
			p = 2*p+1;
		}
		else
		{
			temp = h->heap[2*p+2];
			h->heap[2*p+2] = h->heap[p];
			h->heap[p] = temp;

			/*just telling each vertex where in the heap it currently is*/
			h->heap[2*p+2]->heappos = 2*p+2;
			h->heap[p]->heappos = p;
			p = 2*p+2;		
		}
	}
	ret->heappos = -1;
	return ret;
}

