//Code written by Yonatan Naamad

#include "graph.h"

#ifndef HEAP_H
#define HEAP_H

/* Understanding how a heap can be implemented in an array is not completely trivial.
 * Essentially, the 0th element of the array is the root (min) of the heap. Its children are
 * at indices 1 and 2. In general, an element in the nth position in the heap has children
 * at indices 2n+1 and 2n+2. The index of the parent of node n is floor((n-1)/2).
 * Visually, the a heap with 8 vertices will looks as follows:
 *                          [0]
 *                         /   \
 *                        /     \
 *                      [1]    [2]
 *                      / \    / \
 *                    [3] [4] [5] [6]
 *                    / \
 *                  [7] [8]
 */
typedef struct
{
	Vertex **heap; /*Array of pointers to vertices*/
	int size;
} Heap;

Heap * makeHeap(int numVertices);
void updateVertex(Heap *h, Vertex *v);
void insert(Heap *h, Vertex *v);
Vertex *deleteMin(Heap *h);
#endif
