//Code written by Yonatan Naamad

#include "sys/timeb.h"
#include "time.h"
#include "stdio.h"
#include "stdlib.h"
#include "graph.h"
#include "heap.h"

#define SHOW_OUTPUT 0
#define INFINITY 10000000.0 /*any sufficiently large number should work*/
#define NONE (Vertex *)-1

/*used as the return value for the main algorithm*/
struct AR
{
	double timetaken;
	double mstsize;
};

/*runs Prim's algorithm, and outputs the results if requested
 *returns the total edge weight in the resultant tree*/
struct AR *prim(Graph *g,int numVertices, int output)
{
	double totalweight=0.0, begintime;
	int i;
	Heap *h;
	struct timeb tp;
	struct AR *ret = malloc(sizeof(struct AR));

	/*make and connect the graph*/
	//Graph *g = createGraph(numVertices);
	//connectGraph(g);

	ftime(&tp);
	begintime = tp.time + .001*tp.millitm;

	h = makeHeap(numVertices);

	/*set some initial vertex values*/
	for (i = 0; i < numVertices; i++)
	{
		g->vertices[i]->cost = INFINITY;
		g->vertices[i]->prev = NONE;
		insert(h,g->vertices[i]);
	}

	/*starting vertex*/
	g->vertices[0]->cost = 0;
	updateVertex(h,g->vertices[0]);

	/*essentially the pseudo-code given in the book*/
	while (h->size != 0)
	{
		Vertex *v = deleteMin(h);
		for (i = 0; i < numVertices; i++)
		{
			Vertex *z = g->vertices[i];
			if (i == v->id || z->heappos == -1) continue;
			if (z->cost > v->edges[i]->weight)
			{
				z->cost = v->edges[i]->weight;
				z->prev = v;
				updateVertex(h,z);
			}
		}
	}
	ftime(&tp);
	ret->timetaken = tp.time + .001*tp.millitm - begintime;

	/*calculate total weight and display output if needed*/
	for (i = 0; i < numVertices; i++)
	{
		if (g->vertices[i]->prev != NONE)
		{
			if (output)
				printf("Edge from vertex #%d to vertex #%d with weight %lf\n",  g->vertices[i]->id, g->vertices[i]->prev->id, g->vertices[i]->prev->edges[g->vertices[i]->id]->weight);
			totalweight += g->vertices[i]->edges[g->vertices[i]->prev->id]->weight;
		}
	}

	ret->mstsize = totalweight;

	return ret;
}

int main(int argc, char **argv)
{
	/* main takes 3 command line arguments:
	 * argv[1] = number of vertices in the graph (int)
	 * argv[2] = whether or not to print the tree (0 for no printing, any other int otherwise)
	 * argv[3] = whether or not to print out the average edge weight (0 for no printing, any other int otherwise)
	 * argv[4] = 0 for no time printing, 1 for print algorithm time*/
	double average;
	int numVertices;
	struct AR *p;

	if (argc != 5)
	{
		fprintf(stderr, "Invalid number of arguments");
		return 1;
	}

	numVertices = atoi(argv[1]);

/*make and connect the graph*/
	Graph *g = createGraph(numVertices);
	connectGraph(g);
	p = prim(g,numVertices, atoi(argv[2]));

	average = p->mstsize / (numVertices-1); /*the average is the total weight of the tree divided by the number of edges (numVertices - 1)*/		

	if (atoi(argv[3]))
		printf("Average edge weight in the MST: %lf\n", average);

	if (atoi(argv[4]))
		printf("Time taken: %.3lf\n", p->timetaken);
	return 0;
}
